/* $LAAS: viamPom.c 2010/06/09 15:21:44 mallet $ */

/*
 * Copyright (c) 2007-2008,2010 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 *                                      Anthony Mallet on Mon Mar  5 2007
 */

#include <portLib.h>
#include <stdio.h>
#include <libgen.h>
#include <dirent.h>
#include <err.h>

#include <viam/viamlib.h>

#include "server/viamHeader.h"
#include "shared.h"


/* --- local data ---------------------------------------------------- */

#define POM_FRAME_NAME      "pom%sFramePos"

const POM_SENSOR_POS viam_poszero = {
  .date = 0,
  .sensorToMain = {
    .euler = { 0. /*all zeros*/ },
    .var = { 0. /*all zeros*/ },
  },
  .mainToBase = {
     .euler = { 0. /*all zeros*/ },
     .var = { 0. /*all zeros*/ },
   },
  .mainToOrigin = {
     .euler = { 0. /*all zeros*/ },
     .var = { 0. /*all zeros*/ },
   },
  .VLocal = {
     .euler = { 0. /*all zeros*/ },
     .var = { 0. /*all zeros*/ },
   }
};


/* --- viamPomInit --------------------------------------------------- */

/** Make a new connection to pom.
 */

STATUS
viamPomInit(viam_bank_t bank, int *report)
{
  viam_bankdata_t *d = viam_bank_usrdata(bank);
  char cuid[1024];
  char *path, *pattern, *uid1, *uid2;
  unsigned int id, idmin;
  struct dirent *de;
  DIR *dir;
  int i;

  if (!d) return ERROR;

  for(i=0; i<viam_bank_nimages(bank); i++) {
    viam_image_ref c = viam_bank_image(vh, bank, i);
    viam_imagedata_t *idata = viamIData(bank, i);
    if (!c || !idata) continue;

    idata->pomSeq = -1U;
    idata->pomFrameId = NULL;
    if (d->tags != VIAM_ENABLE) continue;

    /* get pom tags from posters */
    sprintf(cuid, POM_FRAME_NAME, viam_image_name(c));
    if (posterFind(cuid, &idata->pomFrameId) == OK) {
      warnx("image `%s' reading pom positions from poster %s",
	    viam_image_name(c), cuid);
      continue;
    }

    /* if a pom poster cannot be found, fall back to files matching the camera
     * uid. This can be used in conjunction with the file: driver. */
    warnx("no pom poster %s for image `%s', trying files instead",
	  cuid, viam_image_name(c));

    strncpy(cuid, viam_camera_uid(viam_bank_camera(vh, bank, i)),
	    sizeof(cuid) - 1);
    path = strrchr(cuid, '.');
    if (path) strcpy(path, ".pos"); else strcat(cuid, ".pos");

    uid1 = strdup(cuid);
    if (uid1) uid2 = strdup(cuid);
    if (!uid1 || !uid2) {
      *report = S_viam_MEMORY;
      return ERROR;
    }

    path = dirname(uid1);
    pattern = basename(uid2);
    dir = opendir(path);
    if (!dir) {
      warnx("no pos file matching %s for image `%s'", cuid, viam_image_name(c));
      free(uid1); free(uid2);
      *report = S_viam_FILE_IO;
      return ERROR;
    }

    idmin = -1U;
    for(de=readdir(dir); de; de=readdir(dir)) {
      if (sscanf(de->d_name, pattern, &id) != 1) continue;

      if (id < idmin) idmin = id;
    }

    closedir(dir);
    free(uid1); free(uid2);

    /* bail out if nothing found */
    if (idmin == -1U) {
      warnx("no pom position file matching `%s'", cuid);
      *report = S_viam_FILE_IO;
      return ERROR;
    }

    /* store current sequence number in image data */
    idata->pomSeq = idmin;
    warnx("image `%s' reading position from file %s", viam_image_name(c), cuid);
  }

  return OK;
}


/* --- viamPomGetTags ------------------------------------------------ */

/** Read the frame position in pom poster */

int
viamPomGetTags(viam_bank_t bank)
{
  viam_bankdata_t *d = viam_bank_usrdata(bank);
  viam_imagedata_t *idata;
  STATUS s;
  int i;

  if (!d) return S_viam_NOT_CONFIGURED;
  if (d->tags != VIAM_ENABLE) return 0;

  for(i=0; i<viam_bank_nimages(bank); i++) {
    idata = viamIData(bank, i);
    if (!idata) continue;

    if (idata->pomSeq != -1U) {
      /* read pom from file */
      char cuid[1024];
      char *ext;

      snprintf(cuid, sizeof(cuid),
	       viam_camera_uid(viam_bank_camera(vh, bank, i)),
	       idata->pomSeq++);
      ext = strrchr(cuid, '.');
      if (ext) strcpy(ext, ".pos"); else strcat(cuid, ".pos");

      if (pomReadSensorPos(cuid, &idata->pomFrame)) {
	warnx("unreadable pom file: `%s'", cuid);
	return S_viam_NO_SUCH_POM_FRAME;
      }

      continue;
    }

    if (idata->pomFrameId) {
      /* read pom from poster */
      s = viamPOM_SENSOR_POSPosterRead(idata->pomFrameId, &idata->pomFrame);
      if (s != OK) {
	warnx("unreadable pom frame: `%s'",
	      viam_image_name(viam_bank_image(vh, bank, i)));
	return S_viam_NO_SUCH_POM_FRAME;
      }

      continue;
    }
  }

  return 0;
}
